unit AndroidGetInetAddress;

interface

{$IFDEF Android}
uses
  System.SysUtils, {System.Types, System.UITypes, System.Classes, System.Variants,}
  IdStack{, IdBaseComponent, IdComponent, IdUDPBase, IdUDPClient, IdGlobal};

procedure GetLocalAddressList(const AAddresses: TIdStackLocalAddressList);
{$ENDIF}

implementation

{$IFDEF Android}

uses
  Androidapi.JNI.Java.Net, Androidapi.JNI.JavaTypes, Androidapi.Helpers,  Androidapi.JNIBridge;

{ Dumb conversion of number of bits to 4-byte SubnetMask string}
function SubnetMask(ACode: string): string;
begin
  var NCode := ACode.ToInteger;
  if NCode < 25 then
    Result := '255.255.255.' + (%11111111 shr NCode).ToString
  else if NCode < 33 then
    Result := '255.255.' + (%11111111 shr NCode-24).ToString + '.0'
  else if NCode < 41 then
    Result := '255.' + (%11111111 shr NCode-32).ToString + '.0.0'
  else if NCode < 49 then
    Result := (%11111111 shr NCode-40).ToString + '.0.0.0'
  else
    Result := '0.0.0.0.';
end;

procedure GetLocalAddressList(const AAddresses: TIdStackLocalAddressList);

begin
  AAddresses.Clear;
  var LInterfaces := TJNetworkInterface.JavaClass.getNetworkInterfaces;

  while LInterfaces.hasMoreElements do
  begin
    var LInterface := TJNetworkInterface.Wrap(TAndroidHelper.JObjectToID(LInterfaces.nextElement));

////// method getInetAddresses evidently does not return a subnet mask
//    var LAddresses := LInterface.getInetAddresses;
//    while LAddresses.hasMoreElements do
//    begin
//      var LAddress := TJInetAddress.Wrap(TAndroidHelper.JObjectToID(LAddresses.nextElement));
//      WriteLog('DEBUG', 'getAddress: ' + JStringToString(LAddress.ToString));
//      if LAddress.isLoopbackAddress then
//        Continue;
//      // Dave Nottage: Hack until I can find out how to check properly
//      WriteLog('DEBUG', 'LName: ' + JStringToString(LAddress.getClass.getName));
//      WriteLog('DEBUG', 'LHostAddress: ' + JStringToString(LAddress.getHostAddress));
//    end;

////// method getInterfaceAddresses does return IP address, mask, and '[null]' or broadcast IP
    var LInterfaceAddresses := LInterface.getInterfaceAddresses;
    for var i := 0 to LInterfaceAddresses.size-1 do
    begin
      var LHostAddress := JStringToString(LInterfaceAddresses.get(i).toString);
//      WriteLog('DEBUG', 'Address ' + i.ToString + ': ' + LHostAddress);
      if not LHostAddress.Contains('[null]') then
      // Have IPv4Inet address with subnet mask and broadcast address
      begin
        var Address := LHostAddress.Split(['/','[',']',' '], TStringSplitOptions.ExcludeEmpty);
        TIdStackLocalAddressIPv4.Create(AAddresses, Address[0], SubnetMask(Address[1]))
      end;
    end;
  end;
end;
{$ENDIF}

end