//********************************************************************************************************************************
//*                                                                                                                              *
//*     TCP Server-Client Library 1.0.40.80  3delite 2013-2025                                                                  *
//*     See TCP Server-Client Library ReadMe.txt for details.                                                                    *
//*                                                                                                                              *
//* Licenses available for usage of this component:                                                                              *
//* Freeware License: 25                                                                                                        *
//* Shareware License: 50                                                                                                       *
//* Commercial License: 250                                                                                                     *
//*                                                                                                                              *
//*     https://www.3delite.hu/Object%20Pascal%20Developer%20Resources/register.html#tcpscl                                      *
//*                                                                                                                              *
//* Home page:                                                                                                                   *
//*                                                                                                                              *
//*     https://www.3delite.hu/Object%20Pascal%20Developer%20Resources/TCPServerClientLibrary.html                               *
//*                                                                                                                              *
//* For other Delphi components see the home page:                                                                               *
//*                                                                                                                              *
//*     https://www.3delite.hu/                                                                                                  *
//*                                                                                                                              *
//* If you have any questions or enquiries please mail: 3delite@3delite.hu                                                       *
//*                                                                                                                              *
//* Good coding! :)                                                                                                              *
//* 3delite                                                                                                                      *
//********************************************************************************************************************************

unit HashFunctions;

interface

Uses
    Classes;

    function HashMD5String(Text: String): String;

    function HashSHA512String(Text: String): String;
    function HashSHA512Stream(Stream: TStream): String;
    function HashSHA512File(FileName: String): String;

    function HashSHA256String(Text: String): String;
    function HashSHA256Stream(Stream: TStream): String;
    function HashSHA256File(FileName: String): String;

    function LoadSSLOpenSSL: Boolean;

var
    HashFunctionsOpenSSLLoaded: Boolean = False;

implementation

Uses
  SysUtils,
  IdHashMessageDigest,
  idHash,
  IdHashSHA,
  IdSSLOpenSSLHeaders
  {$IF DEFINED(IOS) and DEFINED(CPUARM) AND DEFINED(TCPSCL_OPENSSSL_IOS)}
  , IdSSLOpenSSLHeaders_static
  {$IFEND}
  ;

function LoadSSLOpenSSL: Boolean;
begin
    HashFunctionsOpenSSLLoaded := IdSSLOpenSSLHeaders.Load;
    Result := HashFunctionsOpenSSLLoaded;
end;

//returns MD5 has for a file
function HashMD5String(Text: String): String;
var
   idmd5: TIdHashMessageDigest5;
   //hash: T4x4LongWordRecord;
begin
   idmd5 := TIdHashMessageDigest5.Create;
   try
        result := idmd5.HashStringAsHex(Text);
   finally
        idmd5.Free;
   end;
end;

function HashSHA512String(Text: String): String;
var
    IdHashSHA512: TIdHashSHA512;
begin
    Result := '';
    if NOT HashFunctionsOpenSSLLoaded then begin
        if NOT LoadSSLOpenSSL then begin
            Exit;
        end;
    end;
    if TIdHashSHA512.IsAvailable then begin
        IdHashSHA512 := TIdHashSHA512.Create;
        try
            Result := IdHashSHA512.HashStringAsHex(Text);
        finally
            FreeAndNil(IdHashSHA512);
        end;
    end;
end;

function HashSHA512Stream(Stream: TStream): String;
var
    IdHashSHA512: TIdHashSHA512;
begin
    Result := '';
    if NOT HashFunctionsOpenSSLLoaded then begin
        if NOT LoadSSLOpenSSL then begin
            Exit;
        end;
    end;
    if TIdHashSHA512.IsAvailable then begin
        IdHashSHA512 := TIdHashSHA512.Create;
        try
            Result := IdHashSHA512.HashStreamAsHex(Stream);
        finally
            FreeAndNil(IdHashSHA512);
        end;
    end;
end;

function HashSHA512File(FileName: String): String;
var
    FileStream: TFileStream;
begin
    Result := '';
    if NOT HashFunctionsOpenSSLLoaded then begin
        if NOT LoadSSLOpenSSL then begin
            Exit;
        end;
    end;
    if TIdHashSHA512.IsAvailable then begin
        try
            FileStream := TFileStream.Create(FileName, fmOpenRead);
            try
                Result := HashSHA512Stream(FileStream);
            finally
                FreeAndNil(FileStream);
            end;
        except
            //*
        end;
    end;
end;

function HashSHA256String(Text: String): String;
var
    IdHashSHA256: TIdHashSHA256;
begin
    Result := '';
    if NOT HashFunctionsOpenSSLLoaded then begin
        if NOT LoadSSLOpenSSL then begin
            Exit;
        end;
    end;
    if TIdHashSHA256.IsAvailable then begin
        IdHashSHA256 := TIdHashSHA256.Create;
        try
            Result := IdHashSHA256.HashStringAsHex(Text);
        finally
            FreeAndNil(IdHashSHA256);
        end;
    end;
end;

function HashSHA256Stream(Stream: TStream): String;
var
    IdHashSHA256: TIdHashSHA256;
begin
    Result := '';
    if NOT HashFunctionsOpenSSLLoaded then begin
        if NOT LoadSSLOpenSSL then begin
            Exit;
        end;
    end;
    if TIdHashSHA256.IsAvailable then begin
        IdHashSHA256 := TIdHashSHA256.Create;
        try
            Result := IdHashSHA256.HashStreamAsHex(Stream);
        finally
            FreeAndNil(IdHashSHA256);
        end;
    end;
end;

function HashSHA256File(FileName: String): String;
var
    FileStream: TFileStream;
begin
    Result := '';
    if NOT HashFunctionsOpenSSLLoaded then begin
        if NOT LoadSSLOpenSSL then begin
            Exit;
        end;
    end;
    if TIdHashSHA256.IsAvailable then begin
        try
            FileStream := TFileStream.Create(FileName, fmOpenRead);
            try
                Result := HashSHA256Stream(FileStream);
            finally
                FreeAndNil(FileStream);
            end;
        except
            //*
        end;
    end;
end;

Initialization

    Randomize;

end.
