//********************************************************************************************************************************
//*                                                                                                                              *
//*     TCP Server-Client Library 1.0.40.80  3delite 2013-2025                                                                  *
//*     See TCP Server-Client Library ReadMe.txt for details.                                                                    *
//*                                                                                                                              *
//* Licenses available for usage of this component:                                                                              *
//* Freeware License: 25                                                                                                        *
//* Shareware License: 50                                                                                                       *
//* Commercial License: 250                                                                                                     *
//*                                                                                                                              *
//*     https://www.3delite.hu/Object%20Pascal%20Developer%20Resources/register.html#tcpscl                                      *
//*                                                                                                                              *
//* Home page:                                                                                                                   *
//*                                                                                                                              *
//*     https://www.3delite.hu/Object%20Pascal%20Developer%20Resources/TCPServerClientLibrary.html                               *
//*                                                                                                                              *
//* For other Delphi components see the home page:                                                                               *
//*                                                                                                                              *
//*     https://www.3delite.hu/                                                                                                  *
//*                                                                                                                              *
//* If you have any questions or enquiries please mail: 3delite@3delite.hu                                                       *
//*                                                                                                                              *
//* Good coding! :)                                                                                                              *
//* 3delite                                                                                                                      *
//********************************************************************************************************************************

unit TCPSCLClientList;

interface

{$I TCPSCLCompilerDefines.inc}

Uses
    SysUtils,
    SyncObjs,
    {$IFDEF TCPSCL_GENERICS}
    Generics.Collections,
    {$ENDIF}
    Classes;

type
    TClient = class
        ID: TGUID;
        LoginName: String;
        UserName: String;
        IPAddress: String;
        Data: TObject;
        Destructor Destroy; override;
    end;

    TClientList = class
    private
        FCriticalSection: TCriticalSection;
    public
        Parent: TObject;
        {$IFDEF TCPSCL_GENERICS}
        Clients: TList<TClient>;
        {$ELSE}
        Clients: TList;
        {$ENDIF}
        Constructor Create(_Parent: TObject);
        Destructor Destroy; override;
        function Find(ID: TGUID): TClient;
        procedure AssignUpdate(ClientList: TStringList);
        procedure Lock;
        procedure UnLock;
        procedure Clear;
    end;

implementation

Uses
    TCPServerClientLibrary;

{ TClientList }

procedure TClientList.AssignUpdate(ClientList: TStringList);
var
    i, k: Integer;
    ClientFound: Boolean;
    Client: TClient;
    CheckGUID: TGUID;
begin
    Lock;
    try
        for i := 0 to ClientList.Count - 1 do begin
            CheckGUID := StringToGUID(Copy(ClientList[i], 1, 38));
            //* Check is new client connected
            ClientFound := False;
            for k := 0 to Clients.Count - 1 do begin
                {$IFDEF TCPSCL_GENERICS}
                if Clients[k].ID = CheckGUID then begin
                    ClientFound := True;
                    Break;
                end;
                {$ELSE}
                if (Clients[k] as TClient).ID = CheckGUID then begin
                    ClientFound := True;
                    Break;
                end;
                {$ENDIF}
            end;
            if NOT ClientFound then begin
                Client := TClient.Create;
                Client.ID := CheckGUID;
                Client.LoginName := Copy(ClientList[i], 50, Pos(',USERNAME=', ClientList[i]) - 50);
                Client.UserName := Copy(ClientList[i], Pos(',USERNAME=', ClientList[i]) + 10, Pos(',IPADDRESS=', ClientList[i]) - (Pos(',USERNAME=', ClientList[i]) + 10));
                Client.IPAddress := Copy(ClientList[i], Pos(',IPADDRESS=', ClientList[i]) + 11, Length(ClientList[i]));
                Clients.Add(Client);
                if Assigned((Parent as TTCPServerLibraryClient).OnClientConnected) then begin
                    (Parent as TTCPServerLibraryClient).OnClientConnected(Parent, Client);
                end;
            end;
        end;
        for i := Clients.Count - 1 downto 0 do begin
            //* Check client left
            ClientFound := False;
            for k := 0 to ClientList.Count - 1 do begin
                CheckGUID := StringToGUID(Copy(ClientList[k], 1, 38));
                {$IFDEF TCPSCL_GENERICS}
                if Clients[i].ID = CheckGUID then begin
                    ClientFound := True;
                    Break;
                end;
                {$ELSE}
                if (Clients[i] as TClient).ID = CheckGUID then begin
                    ClientFound := True;
                    Break;
                end;
                {$ENDIF}
            end;
            if NOT ClientFound then begin
                if Assigned((Parent as TTCPServerLibraryClient).OnClientDisconnected) then begin
                    (Parent as TTCPServerLibraryClient).OnClientDisconnected(Parent, Clients[i]);
                end;
                {$IFDEF NEXTGEN}
                Clients[i].DisposeOf;
                {$ELSE}
                Clients[i].Free;
                {$ENDIF}
                Clients.Delete(i);
            end;
        end;
    finally
        UnLock;
    end;
end;

procedure TClientList.Clear;
var
    i: Integer;
begin
    Lock;
    try
        for i := Clients.Count - 1 downto 0 do begin
            {$IFDEF NEXTGEN}
            Clients[i].DisposeOf;
            {$ELSE}
            Clients[i].Free;
            {$ENDIF}
        end;
        Clients.Clear;
    finally
        UnLock;
    end;
end;

constructor TClientList.Create(_Parent: TObject);
begin
    Parent := _Parent;
    inherited Create;
    {$IFDEF TCPSCL_GENERICS}
    Clients := TList<TClient>.Create;
    {$ELSE}
    Clients := TList.Create;
    {$ENDIF}
    FCriticalSection := TCriticalSection.Create;
end;

destructor TClientList.Destroy;
begin
    Clear;
    FreeAndNil(Clients);
    FreeAndNil(FCriticalSection);
    inherited;
end;

function TClientList.Find(ID: TGUID): TClient;
var
    i: Integer;
begin
    Result := nil;
    for i := 0 to Clients.Count - 1 do begin
        if Clients[i].ID = ID then begin
            Result := Clients[i];
            Break;
        end;
    end;
end;

procedure TClientList.Lock;
begin
    FCriticalSection.Enter;
end;

procedure TClientList.UnLock;
begin
    FCriticalSection.Leave;
end;

{ TClient }

destructor TClient.Destroy;
begin
    if Assigned(Data) then begin
        FreeAndNil(Data);
    end;
    inherited;
end;

end.
