//********************************************************************************************************************************
//*                                                                                                                              *
//*     TCP Server-Client Library 1.0.40.80  3delite 2013-2025                                                                  *
//*     See TCP Server-Client Library ReadMe.txt for details.                                                                    *
//*                                                                                                                              *
//* Licenses available for usage of this component:                                                                              *
//* Freeware License: 25                                                                                                        *
//* Shareware License: 50                                                                                                       *
//* Commercial License: 250                                                                                                     *
//*                                                                                                                              *
//*     https://www.3delite.hu/Object%20Pascal%20Developer%20Resources/register.html#tcpscl                                      *
//*                                                                                                                              *
//* Home page:                                                                                                                   *
//*                                                                                                                              *
//*     https://www.3delite.hu/Object%20Pascal%20Developer%20Resources/TCPServerClientLibrary.html                               *
//*                                                                                                                              *
//* For other Delphi components see the home page:                                                                               *
//*                                                                                                                              *
//*     https://www.3delite.hu/                                                                                                  *
//*                                                                                                                              *
//* If you have any questions or enquiries please mail: 3delite@3delite.hu                                                       *
//*                                                                                                                              *
//* Good coding! :)                                                                                                              *
//* 3delite                                                                                                                      *
//********************************************************************************************************************************

unit TCPServerClientLibrary;

interface

{$I TCPSCLCompilerDefines.inc}

Uses
    Classes,
    SysUtils,
    IdContext,
    IdSocketHandle,
    TCPLibraryClasses,
    TCPServerThread,
    TCPClientThread,
    ThreadMessageQueue,
    UsersList,
    TCPSCLClientList;

Const
    STR_SERVER_NAME = '3delite''s TCP Server-Client Library Server';

type
    [ComponentPlatformsAttribute(pidWin32 OR pidWin64 OR pidOSX32 {$IF CompilerVersion >= 33}OR pidiOSSimulator32 OR pidiOSDevice32 OR pidAndroidArm32 OR pidLinux64{$ELSE} {$IF CompilerVersion >= 26}OR pidiOSSimulator OR pidiOSDevice OR pidAndroid {$IFEND} {$IFEND})]
    TTCPServerLibraryServer = class (TComponent)
    private
        FServerName: String;
        FDefaultPort: Cardinal;
        FBindings: TIdSocketHandles;
        FMaxConnections: Cardinal;
        FOnServerResultMessage: TServerResultMessageEvent;
        FOnServerCommandReceive: TServerCommandReceiveEvent;
        FOnServerClientConnected: TServerClientConnectedEvent;
        FOnServerClientDisconnected: TServerClientDisconnectedEvent;
        FOnServerLoginQuery: TServerLoginQueryEvent;
        FOnServerBeforeActivate: TServerBeforeActivateEvent;
        FOnServerAfterActivate: TServerAfterActivateEvent;
        FOnServerBeforeDeActivate: TServerBeforeDeActivateEvent;
        FOnServerAfterDeActivate: TServerAfterDeActivateEvent;
        FOnServerThreadStart: TServerThreadStartEvent;
        FOnServerThreadEnd: TServerThreadEndEvent;
        FOnServerCustomSend: TServerCustomSendEvent;
        FLoggingOn: Boolean;
        FSSLTLSUse: Boolean;
        FSSLPassword: String;
        FSSLRootCertFile: String;
        FSSLCertFile: String;
        FSSLKeyFile: String;
        FLoginHashingMode: TTCPSCLHashingMode;
        FSendUserListToClients: Boolean;
        FParameters: TTCPServerParams;
        function GetActive: Boolean;
        procedure SetBindings(const AValue: TIdSocketHandles);
        procedure SetSendUserListToClients(Value: Boolean);
    public
        TCPServerThread: TTCPServerThread;
        ActionMessageQueue: TThreadMessageQueue;
        IncomingMessageQueue: TThreadMessageQueue;
        LoginList: TLoginList;
        User: TObject;
        Constructor Create(AOwner: TComponent); override;
        Destructor Destroy; override;
        procedure Start;
        procedure Stop;
        procedure CheckIncomingMessages;
    published
        property ServerName: String read FServerName write FServerName;
        property DefaultPort: Cardinal read FDefaultPort write FDefaultPort;
        property Bindings: TIdSocketHandles read FBindings write SetBindings;
        property MaxConnections: Cardinal read FMaxConnections write FMaxConnections;
        property Active: Boolean read GetActive;
        property OnReceiveMessage: TServerResultMessageEvent read FOnServerResultMessage write FOnServerResultMessage;
        property OnReceive: TServerCommandReceiveEvent read FOnServerCommandReceive write FOnServerCommandReceive;
        property OnClientConnected: TServerClientConnectedEvent read FOnServerClientConnected write FOnServerClientConnected;
        property OnClientDisconnected: TServerClientDisconnectedEvent read FOnServerClientDisconnected write FOnServerClientDisconnected;
        property OnLoginQuery: TServerLoginQueryEvent read FOnServerLoginQuery write FOnServerLoginQuery;
        property OnServerBeforeActivate: TServerBeforeActivateEvent read FOnServerBeforeActivate write FOnServerBeforeActivate;
        property OnServerAfterActivate: TServerAfterActivateEvent read FOnServerAfterActivate write FOnServerAfterActivate;
        property OnServerBeforeDeActivate: TServerBeforeDeActivateEvent read FOnServerBeforeDeActivate write FOnServerBeforeDeActivate;
        property OnServerAfterDeActivate: TServerAfterDeActivateEvent read FOnServerAfterDeActivate write FOnServerAfterDeActivate;
        property OnServerThreadStart: TServerThreadStartEvent read FOnServerThreadStart write FOnServerThreadStart;
        property OnServerThreadEnd: TServerThreadEndEvent read FOnServerThreadEnd write FOnServerThreadEnd;
        property OnCustomSend: TServerCustomSendEvent read FOnServerCustomSend write FOnServerCustomSend;
        property LoggingOn: Boolean read FLoggingOn write FLoggingOn default True;
        property SSLTLSUse: Boolean read FSSLTLSUse write FSSLTLSUse default False;
        property SSLPassword: String read FSSLPassword write FSSLPassword;
        property SSLRootCertFile: String read FSSLRootCertFile write FSSLRootCertFile;
        property SSLCertFile: String read FSSLCertFile write FSSLCertFile;
        property SSLKeyFile: String read FSSLKeyFile write FSSLKeyFile;
        property LoginHashingMode: TTCPSCLHashingMode read FLoginHashingMode write FLoginHashingMode default hmMD5;
        property SendUserListToClients: Boolean read FSendUserListToClients write SetSendUserListToClients default False;
        property Parameters: TTCPServerParams read FParameters write FParameters;
    end;

type
    [ComponentPlatformsAttribute(pidWin32 OR pidWin64 OR pidOSX32 {$IF CompilerVersion >= 33}OR pidiOSSimulator32 OR pidiOSDevice32 OR pidAndroidArm32 OR pidLinux64{$ELSE} {$IF CompilerVersion >= 26}OR pidiOSSimulator OR pidiOSDevice OR pidAndroid {$IFEND} {$IFEND})]
    TTCPServerLibraryClient = class (TComponent)
    private
        FHost: String;
        FPort: Cardinal;
        FLoginName: String;
        FPassWord: String;
        FUserName: String;
        FOnResultMessage: TClientResultMessageEvent;
        FOnCommandReceive: TClientCommandReceiveEvent;
        FOnConnected: TClientConnectedEvent;
        FOnDisconnected: TClientDisconnectedEvent;
        FOnCustomSend: TClientCustomSendEvent;
        FLoggingOn: Boolean;
        FSSLTLSUse: Boolean;
        FLoginHashingMode: TTCPSCLHashingMode;
        FOnClientConnected: TClientConnectionChangeEvent;
        FOnClientDisconnected: TClientConnectionChangeEvent;
        FParameters: TTCPClientParams;
        function GetActive: Boolean;
    public
        TCPClientThread: TTCPClientThread;
        ActionMessageQueue: TThreadMessageQueue;
        IncomingMessageQueue: TThreadMessageQueue;
        User: TObject;
        ClientList: TClientList;
        Constructor Create(AOwner: TComponent); override;
        Destructor Destroy; override;
        procedure Connect;
        procedure Disconnect;
        procedure CheckIncomingMessages;
        procedure ProcessIncomingClientList(MessageClientList: TMessageBinaryMemory);
    published
        property Host: String read FHost write FHost;
        property Port: Cardinal read FPort write FPort;
        property LoginName: String read FLoginName write FLoginName;
        property PassWord: String read FPassWord write FPassWord;
        property UserName: String read FUserName write FUserName;
        property Active: Boolean read GetActive;
        property OnReceiveMessage: TClientResultMessageEvent read FOnResultMessage write FOnResultMessage;
        property OnReceive: TClientCommandReceiveEvent read FOnCommandReceive write FOnCommandReceive;
        property OnConnected: TClientConnectedEvent read FOnConnected write FOnConnected;
        property OnDisconnected: TClientDisconnectedEvent read FOnDisconnected write FOnDisconnected;
        property OnCustomSend: TClientCustomSendEvent read FOnCustomSend write FOnCustomSend;
        property LoggingOn: Boolean read FLoggingOn write FLoggingOn default True;
        property SSLTLSUse: Boolean read FSSLTLSUse write FSSLTLSUse default False;
        property LoginHashingMode: TTCPSCLHashingMode read FLoginHashingMode write FLoginHashingMode default hmMD5;
        property OnClientConnected: TClientConnectionChangeEvent read FOnClientConnected write FOnClientConnected;
        property OnClientDisconnected: TClientConnectionChangeEvent read FOnClientDisconnected write FOnClientDisconnected;
        property Parameters: TTCPClientParams read FParameters write FParameters;
    end;

    procedure Register;

implementation

procedure Register;
begin
    RegisterComponents('3delite', [TTCPServerLibraryServer, TTCPServerLibraryClient]);
end;

{ TTCPServerLibraryServer }

procedure TTCPServerLibraryServer.CheckIncomingMessages;
var
    MessageCount: Integer;
    TCPServerMessage: TTCPServerMessage;
begin
    repeat
        MessageCount := IncomingMessageQueue.Count;
        if MessageCount > 0 then begin
            TCPServerMessage := TTCPServerMessage(IncomingMessageQueue.Pop);
            if TCPServerMessage is TMessageTerminated then begin
                Stop;
            end;
            if Assigned(FOnServerResultMessage) then begin
                FOnServerResultMessage(Self, TCPServerMessage);
            end else begin
                FreeAndNil(TCPServerMessage);
            end;
        end;
    until MessageCount < 2;
end;

constructor TTCPServerLibraryServer.Create(AOwner: TComponent);
begin
    inherited Create(AOwner);
    ActionMessageQueue := TThreadMessageQueue.Create;
    IncomingMessageQueue := TThreadMessageQueue.Create;
    FBindings := TIdSocketHandles.Create(Self);
    LoginList := TLoginList.Create;
    FServerName := STR_SERVER_NAME;
    FDefaultPort := 2013;
    FMaxConnections := 100;
    FLoginHashingMode := hmMD5;
    FParameters := TTCPServerParams.Create;
end;

destructor TTCPServerLibraryServer.Destroy;
begin
    Stop;
    FreeAndNil(ActionMessageQueue);
    FreeAndNil(IncomingMessageQueue);
    FreeAndNil(LoginList);
    FreeAndNil(FBindings);
    FreeAndNil(FParameters);
    if Assigned(User) then begin
        FreeAndNil(User);
    end;
    inherited;
end;

function TTCPServerLibraryServer.GetActive: Boolean;
begin
    Result := Assigned(TCPServerThread);
end;

procedure TTCPServerLibraryServer.SetBindings(const AValue: TIdSocketHandles);
begin
    FBindings.Clear;
    if Assigned(AValue) then begin
        FBindings.Assign(AValue);
    end;
end;

procedure TTCPServerLibraryServer.SetSendUserListToClients(Value: Boolean);
begin
    FSendUserListToClients := Value;
    if Assigned(TCPServerThread) then begin
        TCPServerThread.SendUserListToClients := FSendUserListToClients;
    end;
end;

procedure TTCPServerLibraryServer.Start;
var
    TCPServerParameters: TTCPServerParameters;
begin
    Stop;
    TCPServerParameters.Parent := Self;
    TCPServerParameters.ServerName := FServerName;
    TCPServerParameters.DefaultPort := FDefaultPort;
    TCPServerParameters.Bindings := FBindings;
    TCPServerParameters.MaxConnections := FMaxConnections;
    TCPServerParameters.LoginList := LoginList;
    TCPServerParameters.LoggingOn := FLoggingOn;
    TCPServerParameters.SSLTLSUse := FSSLTLSUse;
    TCPServerParameters.SSLPassword := FSSLPassword;
    TCPServerParameters.SSLRootCertFile := FSSLRootCertFile;
    TCPServerParameters.SSLCertFile := FSSLCertFile;
    TCPServerParameters.SSLKeyFile := FSSLKeyFile;
    TCPServerParameters.LoginHashingMode := FLoginHashingMode;
    TCPServerParameters.Parameters := FParameters;
    TCPServerParameters.ActionMessageQueue := ActionMessageQueue;
    TCPServerParameters.IncomingMessageQueue := IncomingMessageQueue;
    TCPServerParameters.OnServerCommandReceive := FOnServerCommandReceive;
    TCPServerParameters.OnServerClientConnected := FOnServerClientConnected;
    TCPServerParameters.OnServerClientDisconnected := FOnServerClientDisconnected;
    TCPServerParameters.OnServerLoginQuery := FOnServerLoginQuery;
    TCPServerParameters.OnServerBeforeActivate := FOnServerBeforeActivate;
    TCPServerParameters.OnServerAfterActivate := FOnServerAfterActivate;
    TCPServerParameters.OnServerBeforeDeActivate := FOnServerBeforeDeActivate;
    TCPServerParameters.OnServerAfterDeActivate := FOnServerAfterDeActivate;
    TCPServerParameters.OnServerThreadStart := FOnServerThreadStart;
    TCPServerParameters.OnServerThreadEnd := FOnServerThreadEnd;
    TCPServerParameters.OnServerCustomSend := FOnServerCustomSend;
    TCPServerParameters.SendUserListToClients := SendUserListToClients;
    TCPServerThread := TTCPServerThread.Create(TCPServerParameters);
end;

procedure TTCPServerLibraryServer.Stop;
begin
    if Assigned(TCPServerThread) then begin
        TCPServerThread.Terminate;
        TCPServerThread.WaitFor;
        FreeAndNil(TCPServerThread);
    end;
end;

{ TTCPServerLibraryClient }

procedure TTCPServerLibraryClient.CheckIncomingMessages;
var
    MessageCount: Integer;
    TCPClientMessage: TTCPServerMessage;
begin
    repeat
        MessageCount := IncomingMessageQueue.Count;
        if MessageCount > 0 then begin
            {$IFDEF NEXTGEN}
            TCPClientMessage := nil;
            {$ENDIF}
            TCPClientMessage := TTCPServerMessage(IncomingMessageQueue.Pop);
            if TCPClientMessage is TMessageTerminated then begin
                Disconnect;
            end;
            if (TCPClientMessage is TMessageBinaryMemory)
            AND ((TCPClientMessage as TMessageBinaryMemory).DataType = TCP_SOCKET_DATA_TYPE_CLIENT_LIST)
            then begin
                ProcessIncomingClientList(TCPClientMessage as TMessageBinaryMemory);
                FreeAndNil(TCPClientMessage);
            end else if Assigned(FOnResultMessage) then begin
                FOnResultMessage(Self, TCPClientMessage);
            end else begin
                FreeAndNil(TCPClientMessage);
            end;
        end;
    until MessageCount < 2;
end;

procedure TTCPServerLibraryClient.ProcessIncomingClientList(MessageClientList: TMessageBinaryMemory);
var
    List: TStringList;
begin
    List := TStringList.Create;
    try
        MessageClientList.Data.Seek(0, soBeginning);
        List.LoadFromStream(MessageClientList.Data);
        ClientList.AssignUpdate(List);
    finally
        FreeAndNil(List);
    end;
end;

procedure TTCPServerLibraryClient.Connect;
var
    TCPClientParameters: TTCPClientParameters;
begin
    Disconnect;
    TCPClientParameters.Parent := Self;
    TCPClientParameters.Host := FHost;
    TCPClientParameters.Port := FPort;
    TCPClientParameters.LoginName := FLoginName;
    TCPClientParameters.PassWord := FPassWord;
    TCPClientParameters.UserName := FUserName;
    TCPClientParameters.LoggingOn := FLoggingOn;
    TCPClientParameters.SSLTLSUse := FSSLTLSUse;
    TCPClientParameters.LoginHashingMode := FLoginHashingMode;
    TCPClientParameters.Parameters := FParameters;
    TCPClientParameters.ActionMessageQueue := ActionMessageQueue;
    TCPClientParameters.IncomingMessageQueue := IncomingMessageQueue;
    TCPClientParameters.OnCommandReceive := FOnCommandReceive;
    TCPClientParameters.OnConnected := FOnConnected;
    TCPClientParameters.OnDisconnected := FOnDisconnected;
    TCPClientParameters.OnCustomSend := FOnCustomSend;
    TCPClientThread := TTCPClientThread.Create(TCPClientParameters);
end;

constructor TTCPServerLibraryClient.Create(AOwner: TComponent);
begin
    inherited Create(AOwner);
    ActionMessageQueue := TThreadMessageQueue.Create;
    IncomingMessageQueue := TThreadMessageQueue.Create;
    ClientList := TClientList.Create(Self);
    FHost := '127.0.0.1';
    FPort := 2013;
    FLoginName := '3delite';
    FPassWord := 'TestPassword';
    FUserName := '3delite';
    FLoginHashingMode := hmMD5;
    FParameters := TTCPClientParams.Create;
end;

destructor TTCPServerLibraryClient.Destroy;
begin
    Disconnect;
    FreeAndNil(ActionMessageQueue);
    FreeAndNil(IncomingMessageQueue);
    FreeAndNil(ClientList);
    FreeAndNil(FParameters);
    if Assigned(User) then begin
        FreeAndNil(User);
    end;
    inherited;
end;

procedure TTCPServerLibraryClient.Disconnect;
begin
    if Assigned(TCPClientThread) then begin
        TCPClientThread.Terminate;
        TCPClientThread.WaitFor;
        FreeAndNil(TCPClientThread);
    end;
end;

function TTCPServerLibraryClient.GetActive: Boolean;
begin
    Result := Assigned(TCPClientThread);
end;

end.
