//********************************************************************************************************************************
//*                                                                                                                              *
//*     TCP Server-Client Library 1.0.40.80  3delite 2013-2025                                                                  *
//*     See TCP Server-Client Library ReadMe.txt for details.                                                                    *
//*                                                                                                                              *
//* Licenses available for usage of this component:                                                                              *
//* Freeware License: 25                                                                                                        *
//* Shareware License: 50                                                                                                       *
//* Commercial License: 250                                                                                                     *
//*                                                                                                                              *
//*     https://www.3delite.hu/Object%20Pascal%20Developer%20Resources/register.html#tcpscl                                      *
//*                                                                                                                              *
//* Home page:                                                                                                                   *
//*                                                                                                                              *
//*     https://www.3delite.hu/Object%20Pascal%20Developer%20Resources/TCPServerClientLibrary.html                               *
//*                                                                                                                              *
//* For other Delphi components see the home page:                                                                               *
//*                                                                                                                              *
//*     https://www.3delite.hu/                                                                                                  *
//*                                                                                                                              *
//* If you have any questions or enquiries please mail: 3delite@3delite.hu                                                       *
//*                                                                                                                              *
//* Good coding! :)                                                                                                              *
//* 3delite                                                                                                                      *
//********************************************************************************************************************************

unit ThreadMessageQueue;

interface

{$I TCPSCLCompilerDefines.inc}

Uses
    SysUtils,
    SyncObjs,
    {$IFDEF TCPSCL_MOBILE}
    Generics.Collections;
    {$ELSE}
    Contnrs;
    {$ENDIF}

type
    TThreadMessageQueue = class
    private
        {$IFDEF TCPSCL_MOBILE}
        FTObjectQueue: TObjectQueue<TObject>;
        {$ELSE}
        FTObjectQueue: TObjectQueue;
        {$ENDIF}
        FCriticalSection: TCriticalSection;
    public
        Constructor Create;
        Destructor Destroy; override;
        procedure Add(Obj: TObject);
        function Pop: TObject;
        procedure Clear;
        function Count: Integer;
    end;

implementation

{ TThreadMessageQue }

procedure TThreadMessageQueue.Add(Obj: TObject);
begin
    FCriticalSection.Enter;
    try
        {$IFDEF TCPSCL_MOBILE}
        FTObjectQueue.Enqueue(Obj);
        {$ELSE}
        FTObjectQueue.Push(Obj);
        {$ENDIF}
    finally
        FCriticalSection.Leave;
    end;
end;

procedure TThreadMessageQueue.Clear;
var
    i: Integer;
    Obj: TObject;
begin
    FCriticalSection.Enter;
    try
        for i := FTObjectQueue.Count - 1 downto 0 do begin
            {$IFDEF TCPSCL_MOBILE}
            Obj := FTObjectQueue.Extract;
            if Assigned(Obj) then begin
                Obj.DisposeOf;
            end;
            {$ELSE}
            Obj := FTObjectQueue.Pop;
            if Assigned(Obj) then begin
                FreeAndNil(Obj);
            end;
            {$ENDIF}
        end;
    finally
        FCriticalSection.Leave;
    end;
end;

constructor TThreadMessageQueue.Create;
begin
    {$IFDEF TCPSCL_MOBILE}
    FTObjectQueue := TObjectQueue<TObject>.Create;
    FTObjectQueue.OwnsObjects := False;
    {$ELSE}
    FTObjectQueue := TObjectQueue.Create;
    {$ENDIF}
    FCriticalSection := TCriticalSection.Create;
end;

destructor TThreadMessageQueue.Destroy;
begin
    Clear;
    FreeAndNil(FCriticalSection);
    FreeAndNil(FTObjectQueue);
    inherited;
end;

function TThreadMessageQueue.Pop: TObject;
begin
    FCriticalSection.Enter;
    try
        //Result := nil;
        {$IFDEF TCPSCL_MOBILE}
        Result := FTObjectQueue.Extract;
        {$ELSE}
        Result := FTObjectQueue.Pop;
        {$ENDIF}
    finally
        FCriticalSection.Leave;
    end;
end;

function TThreadMessageQueue.Count: Integer;
begin
    FCriticalSection.Enter;
    try
        Result := FTObjectQueue.Count;
    finally
        FCriticalSection.Leave;
    end;
end;

end.
