//********************************************************************************************************************************
//*                                                                                                                              *
//*     TCP Server-Client Library  3delite 2013-2025                                                                            *
//*     See TCP Server-Client Library ReadMe.txt for details.                                                                    *
//*                                                                                                                              *
//* Licenses available for usage of this component:                                                                              *
//* Freeware License: 25                                                                                                        *
//* Shareware License: 50                                                                                                       *
//* Commercial License: 250                                                                                                     *
//*                                                                                                                              *
//*     https://www.3delite.hu/Object%20Pascal%20Developer%20Resources/register.html#tcpscl                                      *
//*                                                                                                                              *
//* Home page:                                                                                                                   *
//*                                                                                                                              *
//*     https://www.3delite.hu/Object%20Pascal%20Developer%20Resources/TCPServerClientLibrary.html                               *
//*                                                                                                                              *
//* For other Delphi components see the home page:                                                                               *
//*                                                                                                                              *
//*     https://www.3delite.hu/                                                                                                  *
//*                                                                                                                              *
//* If you have any questions or enquiries please mail: 3delite@3delite.hu                                                       *
//*                                                                                                                              *
//* Good coding! :)                                                                                                              *
//* 3delite                                                                                                                      *
//********************************************************************************************************************************

unit Unit1;

interface

{$I TCPSCLCompilerDefines.inc}

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Dialogs, FMX.Edit, FMX.Layouts,
  FMX.Memo, TCPClientThread, TCPLibraryClasses, TCPServerClientLibrary, TCPSCLClientList, FMX.Memo.Types, FMX.StdCtrls, FMX.ScrollBox, FMX.Controls.Presentation;

type
  TForm1 = class(TForm)
    GroupBox1: TGroupBox;
    Memo1: TMemo;
    GroupBox2: TGroupBox;
    EditChat: TEdit;
    SendBn: TButton;
    ConnectBn: TButton;
    DisconnectBn: TButton;
    TimerResults: TTimer;
    TCPServerLibraryClient1: TTCPServerLibraryClient;
    Memo2: TMemo;
    procedure DisconnectBnClick(Sender: TObject);
    procedure ConnectBnClick(Sender: TObject);
    procedure SendBnClick(Sender: TObject);
    procedure TimerResultsTimer(Sender: TObject);
    procedure TCPServerLibraryClient1ReceiveMessage(Sender: TObject; TCPClientMessage: TTCPServerMessage);
    procedure FormCreate(Sender: TObject);
    procedure TCPServerLibraryClient1ClientConnected(Sender: TObject; Client: TClient);
    procedure TCPServerLibraryClient1ClientDisconnected(Sender: TObject; Client: TClient);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure ArrivedConnected(MessageConnected: TMessageConnected);
    procedure ArrivedLog(MessageLog: TMessageLog);
    procedure ArrivedChat(MessageChat: TMessageChat);
    procedure ArrivedBinary(MessageBinary: TMessageBinary);
    procedure ArrivedCommand(MessageCommand: TMessageCommand);
  end;

var
  Form1: TForm1;
  TransferID: Integer;

implementation

{$R *.fmx}

Uses
    IdStack;

procedure TForm1.ConnectBnClick(Sender: TObject);
begin
    //* Use secure SSL TLS connnection
    //TCPServerLibraryClient1.SSLTLSUse := True;
    //* Logging on
    TCPServerLibraryClient1.LoggingOn := True;
    //* To get the default local address
    TIdStack.IncUsage;
    try
        TCPServerLibraryClient1.Host := GStack.LocalAddress;
    finally
        TIdStack.DecUsage;
    end;
    //* Make connecttion
    TCPServerLibraryClient1.Connect;
end;

procedure TForm1.DisconnectBnClick(Sender: TObject);
begin
    TCPServerLibraryClient1.Disconnect;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
    {$IFDEF MSWINDOWS}
    if TCPServerLibraryClient1.LoginHashingMode = hmOpenSSL then begin
        if (NOT FileExists('libeay32.dll'))
        OR (NOT FileExists('ssleay32.dll'))
        then begin
            Showmessage('For the password hashing function to work correctly when using hmOpenSSL on Windows, include ''libeay32.dll'' and ''ssleay32.dll'''
            + '(from Delphi install folder eg. ''C:\Program Files\Embarcadero\RAD Studio\12.0\bin\subversion'') in your installation package when you distribute your software'
            + 'and make sure to use the version according to the Indy version used or else you will get a rejected login always.' + #13#10
            + 'These files are available on the system where Delphi is installed, the mentioned folder is added to the PATH, but not on other systems.'
            );
        end;
    end;
    {$ENDIF}
end;

procedure TForm1.SendBnClick(Sender: TObject);
var
    MessageChat: TMessageChat;
begin
    MessageChat := TMessageChat.Create;
    MessageChat.Text := EditChat.Text;
    //* Send this chat string to the server, this call is thread-safe, it's possible add objects from any thread
    TCPServerLibraryClient1.ActionMessageQueue.Add(MessageChat);
end;

procedure TForm1.TCPServerLibraryClient1ClientConnected(Sender: TObject; Client: TClient);
begin
    Memo2.Lines.Append('New: ' + GUIDToString(Client.ID) + ', ' + Client.LoginName + ', ' + Client.UserName + ', ' + Client.IPAddress);
end;

procedure TForm1.TCPServerLibraryClient1ClientDisconnected(Sender: TObject; Client: TClient);
begin
    Memo2.Lines.Append('Left: ' + GUIDToString(Client.ID) + ', ' + Client.LoginName + ', ' + Client.UserName + ', ' + Client.IPAddress);
end;

procedure TForm1.TCPServerLibraryClient1ReceiveMessage(Sender: TObject; TCPClientMessage: TTCPServerMessage);
begin
    //* Do not free TMessageTransferRequest object - it is sent back if accepted!
    if TCPClientMessage is TMessageTransferRequest then begin
        //ArrivedTransferRequest(TMessageTransferRequest(TCPClientMessage));
        FreeAndNil(TCPClientMessage);
    end else begin
        try
            //* Decide what kind of message has arrived and act accordingly
            if TCPClientMessage is TMessageConnected then begin
                ArrivedConnected(TMessageConnected(TCPClientMessage));
            end;
            if TCPClientMessage is TMessageLog then begin
                ArrivedLog(TMessageLog(TCPClientMessage));
            end;
            if TCPClientMessage is TMessageChat then begin
                ArrivedChat(TMessageChat(TCPClientMessage));
            end;
            if TCPClientMessage is TMessageBinary then begin
                ArrivedBinary(TMessageBinary(TCPClientMessage));
            end;
            if TCPClientMessage is TMessageProgress then begin
                //ArrivedProgress(TMessageProgress(TCPClientMessage));
            end;
            //* If the message is a command...
            if TCPClientMessage is TMessageCommand then begin
                ArrivedCommand(TMessageCommand(TCPClientMessage));
            end;
        finally
            //* Never forget to free the object if finished with it
            FreeAndNil(TCPClientMessage);
        end;
    end;
    //* Breath
    Application.ProcessMessages;
end;

procedure TForm1.TimerResultsTimer(Sender: TObject);
begin
    //* This function needs to be called frequently it triggers the OnReceiveMessage() event if there is at least one message in the incoming message queue. The event OnReceiveMessage() will be called in the same thread as where CheckIncomingMessages() is called.
    TCPServerLibraryClient1.CheckIncomingMessages;
end;

procedure TForm1.ArrivedConnected(MessageConnected: TMessageConnected);
begin
    if MessageConnected.Connected then begin
        ConnectBn.Enabled := False;
        DisconnectBn.Enabled := True;
    end else begin
        ConnectBn.Enabled := True;
        DisconnectBn.Enabled := False;
    end;
end;

procedure TForm1.ArrivedLog(MessageLog: TMessageLog);
begin
    //* Display the log
    Memo1.Lines.Append(MessageLog.Text);
end;

procedure TForm1.ArrivedChat(MessageChat: TMessageChat);
begin
    //* Display the string
    Memo1.Lines.Append({MessageChat.UserName + ': ' + }MessageChat.Text);
end;

procedure TForm1.ArrivedCommand(MessageCommand: TMessageCommand);
begin
    //* Just display the command
    Memo1.Lines.Append('Command from server: ' + IntToStr(MessageCommand.ID) + ' ' + MessageCommand.Command + ' ' + MessageCommand.Arguments);
end;

procedure TForm1.ArrivedBinary(MessageBinary: TMessageBinary);
begin
    //* Display the picture
    if MessageBinary.DataType = TCP_SOCKET_DATA_TYPE_PICTURE then begin
        MessageBinary.Data.Seek(0, soBeginning);
        //ImageControl1.Bitmap.LoadFromStream(MessageBinary.Data);
        Memo1.Lines.Append('Received picture.');
    end;
    //* Just dispplay a notice that the file transfer finished
    if MessageBinary.DataType = TCP_SOCKET_DATA_TYPE_FILE then begin
        Memo1.Lines.Append('Received file: ' + MessageBinary.FileName);
    end;
end;

end.
