//********************************************************************************************************************************
//*                                                                                                                              *
//*     TCP Server-Client Library 1.0.40.80  3delite 2013-2025                                                                  *
//*     See TCP Server-Client Library ReadMe.txt for details.                                                                    *
//*                                                                                                                              *
//* Licenses available for usage of this component:                                                                              *
//* Freeware License: 25                                                                                                        *
//* Shareware License: 50                                                                                                       *
//* Commercial License: 250                                                                                                     *
//*                                                                                                                              *
//*     https://www.3delite.hu/Object%20Pascal%20Developer%20Resources/register.html#tcpscl                                      *
//*                                                                                                                              *
//* Home page:                                                                                                                   *
//*                                                                                                                              *
//*     https://www.3delite.hu/Object%20Pascal%20Developer%20Resources/TCPServerClientLibrary.html                               *
//*                                                                                                                              *
//* For other Delphi components see the home page:                                                                               *
//*                                                                                                                              *
//*     https://www.3delite.hu/                                                                                                  *
//*                                                                                                                              *
//* If you have any questions or enquiries please mail: 3delite@3delite.hu                                                       *
//*                                                                                                                              *
//* Good coding! :)                                                                                                              *
//* 3delite                                                                                                                      *
//********************************************************************************************************************************

unit UsersList;

interface

{$I TCPSCLCompilerDefines.inc}

Uses
    SysUtils,
    SyncObjs,
    {$IFDEF TCPSCL_GENERICS}
    Generics.Collections,
    {$ENDIF}
    {$IFDEF TCPSCL_MOBILE}
    Posix.Unistd,
    {$ENDIF}
    Classes;

    {
type
    TUserRights = record
        Administrator: Boolean;
        Control: Boolean;
    end;
    }

type
    TUserRights = UInt64;

type
    TLogin = class
    public
        Name: String;
        PassWord: String;
        Rights: TUserRights;
        Constructor Create(Name: String; PassWord: String); overload;
        procedure Assign(From: TLogin);
    end;

type
    TLoginList = class
    private
        FCriticalSection: TCriticalSection;
    public
        {$IFDEF TCPSCL_GENERICS}
        List: TList<TLogin>;
        {$ELSE}
        List: TList;
        {$ENDIF}
        Constructor Create;
        Destructor Destroy; override;
        procedure Add(Login: TLogin);
        procedure Delete(Index: Integer);
        function Find(LoginName: String; var Login: TLogin): Boolean;
        procedure Clear;
        function Count: Integer;
        procedure Lock;
        procedure UnLock;
        function Save(const FileName: String): Boolean;
        function Load(const FileName: String): Boolean;
    end;

implementation

Uses
    IniFiles;

function EnDeCrypt(const Value: String): String;
var
    CharIndex : integer;
begin
    Result := Value;
    for CharIndex := 0 to Length(Value) - 1 do begin
        Result[CharIndex] := chr(NOT(ord(Value[CharIndex])));
    end;
end;

{ TThreadMessageQue }

procedure TLoginList.Add(Login: TLogin);
begin
    List.Add(Login);
end;

procedure TLoginList.Clear;
var
    i: Integer;
begin
    for i := List.Count - 1 downto 0 do begin
        TObject(List[i]).{$IFDEF TCPSCL_MOBILE}DisposeOf{$ELSE}Free{$ENDIF};
        List.Delete(i);
    end;
end;

constructor TLoginList.Create;
begin
    {$IFDEF TCPSCL_GENERICS}
    List := TList<TLogin>.Create;
    {$ELSE}
    List := TList.Create;
    {$ENDIF}
    FCriticalSection := TCriticalSection.Create;
end;

procedure TLoginList.Delete(Index: Integer);
begin
    if (Index > - 1)
    AND (Index < List.Count)
    then begin
        TObject(List[Index]).{$IFDEF TCPSCL_MOBILE}DisposeOf{$ELSE}Free{$ENDIF};
        List.Delete(Index);
    end;
end;

destructor TLoginList.Destroy;
begin
    FreeAndNil(FCriticalSection);
    Clear;
    FreeAndNil(List);
    inherited;
end;

function TLoginList.Find(LoginName: String; var Login: TLogin): Boolean;
var
    i: Integer;
begin
    Result := False;
    Login := nil;
    for i := 0 to List.Count - 1 do begin
        if SameText(TLogin(List[i]).Name, LoginName) then begin
            Login := TLogin.Create;
            Login.Assign(TLogin(List[i]));
            Result := True;
            Break;
        end;
    end;
end;

function TLoginList.Load(const FileName: String): Boolean;
var
    i: Integer;
    Ini: TIniFile;
    User: TLogin;
    UserCount: Integer;
begin
    Ini := TIniFile.Create(FileName);
    Lock;
    try
        Clear;
        UserCount := Ini.ReadInteger('Global', 'Count', 0);
        for i := 0 to UserCount - 1 do begin
            User := TLogin.Create;
            User.Name := Ini.ReadString(IntToStr(i), 'Name', 'Anonymous');
            User.PassWord := EnDeCrypt(Ini.ReadString(IntToStr(i), 'Password', ''));
            Add(User);
        end;
        Result := True;
    finally
        UnLock;
        FreeAndNil(Ini);
    end;
end;

function TLoginList.Save(const FileName: String): Boolean;
var
    i: Integer;
    Ini: TIniFile;
    User: TLogin;
begin
    if FileExists(FileName) then begin
        DeleteFile(FileName);
    end;
    Ini := TIniFile.Create(FileName);
    Lock;
    try
        Ini.WriteInteger('Global', 'Count', List.Count);
        for i := 0 to List.Count - 1 do begin
            User := TLogin(List[i]);
            Ini.WriteString(IntToStr(i), 'Name', User.Name);
            Ini.WriteString(IntToStr(i), 'Password', EnDeCrypt(User.PassWord));
        end;
        Result := True;
    finally
        UnLock;
        FreeAndNil(Ini);
    end;
end;

procedure TLoginList.Lock;
begin
    FCriticalSection.Enter;
end;

procedure TLoginList.UnLock;
begin
    FCriticalSection.Leave;
end;

function TLoginList.Count: Integer;
begin
    Result := List.Count;
end;

{ TLogin }

procedure TLogin.Assign(From: TLogin);
begin
    Name := From.Name;
    PassWord := From.PassWord;
    Rights := From.Rights;
end;

constructor TLogin.Create(Name, PassWord: String);
begin
    inherited Create;
    Self.Name := Name;
    Self.PassWord := PassWord;
end;

end.
